//
// Demonstrates the use of DATEVLD.DLL.
// March 30, 1992.
// Copr. (c) 1992 Albert C. Schmitt.  All Rights Reserved.
//
#include <owl.h>
#include <strstream.h>
#include <dos.h>
#include "generic.h"
#include "datevld.h"

class TMyApplication : public TApplication
{
public:
	TMyApplication(LPSTR  AName,
				   HANDLE hInstance,
				   HANDLE hPrevInstance,
				   LPSTR  lpCmdLine,
				   int    nCmdShow) : TApplication(AName,
												   hInstance,
												   hPrevInstance,
												   lpCmdLine,
												   nCmdShow) {};
	virtual void InitMainWindow();
};

_CLASSDEF(TMyWindow)
class TMyWindow : public TWindow
{
public:
	TMyWindow(PTWindowsObject AParent, LPSTR ATitle);
	virtual void Date(RTMessage Msg) = [CM_FIRST + IDM_DATE];
	virtual void About(RTMessage Msg) = [CM_FIRST + IDM_ABOUT];
};

_CLASSDEF(TDateDialog)
class TDateDialog : public TDialog
{
	char datebuf[12];
public:
	TDateDialog(PTWindowsObject AParent,
				LPSTR ATitle) : TDialog(AParent,
										ATitle){};
	virtual void SetupWindow();
	virtual void SetJulian(LONG jDate);
	virtual LONG IncDay(int incr);
	virtual void Ok(RTMessage Msg) = [ID_FIRST + IDOK];
	virtual void Next(RTMessage Msg) = [ID_FIRST + ID_NEXT];
	virtual void Prior(RTMessage Msg) = [ID_FIRST + ID_PRIOR];
};

//--------------------------------------------------
// TMyWindow's method implementations:
//--------------------------------------------------
TMyWindow::TMyWindow(PTWindowsObject AParent,
					 LPSTR ATitle) : TWindow(AParent,
											 ATitle)
{
	AssignMenu("GenericMenu");
}

void TMyWindow::Date(RTMessage /*Msg*/)
{
	GetApplication()->ExecDialog(new TDateDialog(this, "DATEDIALOG"));
}

void TMyWindow::About(RTMessage /*Msg*/)
{
	GetApplication()->ExecDialog(new TDialog(this, "ABOUTBOX"));
}

//--------------------------------------------------
// TDateDialog's method implementations:
//--------------------------------------------------
void TDateDialog::SetupWindow()
{
	struct dosdate_t datep;

	TDialog::SetupWindow();

	_dos_getdate(&datep);
	StrDate((LPSTR)datebuf,
			(WORD)datep.month,
			(WORD)datep.day,
			(WORD)datep.year);

	SetDlgItemText(HWindow, IDC_DATE, (LPSTR)datebuf);
	SetJulian(toJul((LPSTR)datebuf));
}

void TDateDialog::SetJulian(LONG jDate)
{
	LPSTR szDayName[25];
	int nStatus;

	ostrstream os(datebuf, sizeof(datebuf));

    // display the Julian date in dialog
	os << jDate << ends;
    SetDlgItemText(HWindow, IDC_JDATE, (LPSTR)datebuf);

	// always pass IDS_FRIDAY as the resource identifier.
	nStatus = dayNameEx(GetWindowWord(HWindow, GWW_HINSTANCE), IDS_FRIDAY,
						(LPSTR)szDayName, 25, jDate);
	if (nStatus == 0)
	    MessageBox(HWindow, 
	               "Could not load day name from Resource",
	               "Generic", MB_OK | MB_ICONEXCLAMATION);
	else
	{
		// display day of week
		SetDlgItemText(HWindow, IDC_SDATE, (LPSTR)szDayName);
	}
}

LONG TDateDialog::IncDay(int incr)
{
	LONG jDate;

    jDate = toJul((LPSTR)datebuf) + incr;
    toDate((LPSTR)datebuf, jDate);

    return jDate;
}

void TDateDialog::Ok(RTMessage /*Msg*/)
{
	LONG jDate;

	GetDlgItemText(HWindow, IDC_DATE, (LPSTR)datebuf, 11);
	jDate = toJul((LPSTR)datebuf);

    if (jDate == 0L)
        MessageBox(HWindow,
                   "Invalid Date",
                   "Generic",
				   MB_OK | MB_ICONEXCLAMATION);
	else
    {
        toDate((LPSTR)datebuf, jDate);
        SetDlgItemText(HWindow, IDC_DATE, (LPSTR)datebuf);
    }

	SetJulian(jDate);
}

void TDateDialog::Next(RTMessage /*Msg*/)
{
	LONG jDate;

	GetDlgItemText(HWindow, IDC_DATE, (LPSTR)datebuf, 11);
	jDate = IncDay(1);
	SetDlgItemText(HWindow, IDC_DATE, (LPSTR)datebuf);
	SetJulian(jDate);
}

void TDateDialog::Prior(RTMessage /*Msg*/)
{
	LONG jDate;

	GetDlgItemText(HWindow, IDC_DATE, (LPSTR)datebuf, 11);
	jDate = IncDay(-1);
	SetDlgItemText(HWindow, IDC_DATE, (LPSTR)datebuf);
	SetJulian(jDate);
}

//--------------------------------------------------
// TMyApplication's method implementations:
//--------------------------------------------------
void TMyApplication::InitMainWindow()
{
	MainWindow = new TMyWindow(NULL, "Date Conversion Demo");
}

//--------------------------------------------------
// Main program:
//--------------------------------------------------
int PASCAL WinMain(HANDLE hInstance,
				   HANDLE hPrevInstance,
				   LPSTR lpCmdLine,
				   int nCmdShow)
{
	TMyApplication MyApp("GenericWClass",
						 hInstance,
						 hPrevInstance,
						 lpCmdLine,
						 nCmdShow);
	MyApp.Run();
	return MyApp.Status;
}
