//
//	Copr. (C) 1990-1993 A. Schmitt. All Rights Reserved.
//
#define STRICT
#include <windows.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include "datevld.h"

extern BOOL bInitDate;			 // declared in StrDate.C
extern char cDateOrder; 		 // declared in StrDate.C
static int	nBaseYear = 1900;	// base year for century conversion.

static void splitdate (LPSTR lpszDate,
					   LPINT lpnMonth,
					   LPINT lpnDay,
					   LPINT lpnYear)
{
	size_t nPos = 0;
	int i;
	static char szBuf[12];
	int v1, v2, v3;

	//
	// If not initialized then build date format string.
	///
	if (!bInitDate)
	{
		bInitDate = TRUE;
		InitDate();
	}

	//
	// Initialize return values for month, day & year.
	//
	*lpnMonth = *lpnDay = *lpnYear = 0;
	if (!lpszDate)
		return;

	for (i=0; i<3 && *lpszDate; i++)
	{
		//
		// skip to next separator
		//
		nPos = 0;
		while (isdigit(*(lpszDate+nPos)))
			nPos++;

		nPos = min(nPos, 10);

		//
		// get numeric value into buffer and increment date string.
		//
		_fstrncpy(szBuf, lpszDate, nPos);
		szBuf[nPos] = '\0';
		lpszDate += nPos;
		if (*lpszDate != '\0')
			lpszDate++;

		//
		// store month, day & year.
		//
		switch (i)
		{
			case 0:
				v1 = atoi(szBuf);
				break;
			case 1:
				v2 = atoi(szBuf);
				break;
			case 2:
				v3 = atoi(szBuf);
				break;

		} // end switch

	} // end for

	//
	// put month, day, year in MDY, DMY or YMD order based upon order
	// specified in [Intl] section of WIN.INI.
	//
	if (cDateOrder == 'M')
	{
		*lpnMonth = v1;
		*lpnDay   = v2;
		*lpnYear  = v3;
	}
	else if (cDateOrder == 'd')
	{
		*lpnDay   = v1;
		*lpnMonth = v2;
		*lpnYear  = v3;
	}
	else
	{
		*lpnYear  = v1;
		*lpnMonth = v2;
		*lpnDay   = v3;
	}
}

//
// sets the base century for date conversions.	Returns the previous
// base year.  Range 15 >= nNewBaseYear <= 99.
//
int FAR PASCAL SetBaseYear(int nNewBaseYear)
{
	int nPrevBaseYear = nBaseYear;

	if (nNewBaseYear < 15 || nNewBaseYear > 99)
		return -1;

	nBaseYear = nNewBaseYear * 100;
	return nPrevBaseYear / 100;
}

//
// accepts date as a string and converts it to julian.
//
LONG FAR PASCAL toJul(LPSTR lpszDate)
{
	int  nMonth,  nDay,  nYear;
	int  nMonth2, nDay2, nYear2;
	long lJDate;

	splitdate (lpszDate, &nMonth, &nDay, &nYear);

	if (nYear < 0)
		return 0L;

	if (nYear < 100)
		nYear += nBaseYear;
	else if (nYear < 1000)
		nYear += 1000;

    //
    // determine julian date, convert it back to mm, dd, yy and check if
	// the values match.  If they don't then the date in
	// lpszDate was invalid.
    //
	lJDate = _tojulian (nMonth, nDay, nYear);
	_fromJulian (&nMonth2, &nDay2, &nYear2, lJDate);

	if ((nMonth == nMonth2) && (nDay == nDay2) && (nYear == nYear2) && lJDate)
		return lJDate;
	else
		return 0L;
}

